import React, { useEffect } from "react";
import classNames from "classnames";
function getPathStyles(offset, percent, strokeColor, strokeWidth, gapDegree, gapPosition, notransition) {
    if (strokeColor === void 0) { strokeColor = "#2db7f5"; }
    if (strokeWidth === void 0) { strokeWidth = 1; }
    if (gapDegree === void 0) { gapDegree = 0; }
    if (gapPosition === void 0) { gapPosition = "top"; }
    if (notransition === void 0) { notransition = false; }
    var radius = 50 - strokeWidth / 2;
    var beginPositionX = 0;
    var beginPositionY = -radius;
    var endPositionX = 0;
    var endPositionY = -2 * radius;
    switch (gapPosition) {
        case "left":
            beginPositionX = -radius;
            beginPositionY = 0;
            endPositionX = 2 * radius;
            endPositionY = 0;
            break;
        case "right":
            beginPositionX = radius;
            beginPositionY = 0;
            endPositionX = -2 * radius;
            endPositionY = 0;
            break;
        case "bottom":
            beginPositionY = radius;
            endPositionY = 2 * radius;
            break;
        default:
    }
    var pathString = "M 50,50 m " + beginPositionX + "," + beginPositionY + "\n     a " + radius + "," + radius + " 0 1 1 " + endPositionX + "," + -endPositionY + "\n     a " + radius + "," + radius + " 0 1 1 " + -endPositionX + "," + endPositionY;
    var len = Math.PI * 2 * radius;
    var pathStyle = {
        stroke: strokeColor,
        strokeDasharray: (percent / 100) * (len - gapDegree) + "px " + len + "px",
        strokeDashoffset: "-" + (gapDegree / 2 + (offset / 100) * (len - gapDegree)) + "px",
        transition: notransition ? null : "stroke-dashoffset .3s ease 0s, stroke-dasharray .3s ease 0s, stroke .3s, stroke-width .06s ease .3s" // eslint-disable-line
    };
    return {
        pathString: pathString,
        pathStyle: pathStyle
    };
}
/**
 * 原型进度条
 * @see https://github.com/react-component/progress/blob/master/src/Circle.js
 */
function Circle(props) {
    var _a = props.prefixCls, prefixCls = _a === void 0 ? "xy-progress-circle" : _a, className = props.className, style = props.style, _b = props.strokeWidth, strokeWidth = _b === void 0 ? 1 : _b, _c = props.strokeColor, strokeColor = _c === void 0 ? "#2db7f5" : _c, _d = props.trailWidth, trailWidth = _d === void 0 ? 1 : _d, _e = props.trailColor, trailColor = _e === void 0 ? "#D9D9D9" : _e, _f = props.strokeLinecap, strokeLinecap = _f === void 0 ? "round" : _f, _g = props.percent, percent = _g === void 0 ? 0 : _g, _h = props.gapDegree, gapDegree = _h === void 0 ? 0 : _h, _j = props.gapPosition, gapPosition = _j === void 0 ? "top" : _j, _k = props.notransition, notransition = _k === void 0 ? false : _k;
    var _l = getPathStyles(0, 100, trailColor, strokeWidth, gapDegree, gapPosition, notransition), pathString = _l.pathString, pathStyle = _l.pathStyle;
    function getStokeList() {
        var percentList = Array.isArray(percent) ? percent : [percent];
        var strokeColorList = Array.isArray(strokeColor) ? strokeColor : [strokeColor];
        var stackPtg = 0;
        return percentList.map(function (ptg, index) {
            var color = strokeColorList[index] || strokeColorList[strokeColorList.length - 1];
            var _a = getPathStyles(stackPtg, ptg, color, strokeWidth, gapDegree, gapPosition, notransition), pathString = _a.pathString, pathStyle = _a.pathStyle;
            // pathStyle.opacity = ptg === 0 ? 0 : 1;
            stackPtg += ptg;
            return React.createElement("path", { key: index, className: prefixCls + "-circle-path", d: pathString, strokeLinecap: strokeLinecap, strokeWidth: strokeWidth, fillOpacity: "0", style: pathStyle });
        });
    }
    useEffect(function () { }, [percent]);
    return (React.createElement("svg", { className: classNames(prefixCls, className), style: style, viewBox: "0 0 100 100" },
        React.createElement("path", { className: prefixCls + "-trail", d: pathString, stroke: trailColor, strokeLinecap: strokeLinecap, strokeWidth: trailWidth || strokeWidth, fillOpacity: "0", style: pathStyle }),
        getStokeList().reverse()));
}
export default React.memo(Circle);
//# sourceMappingURL=Circle.js.map