import React, { useRef, useEffect, useState } from "react";
import classNames from "classnames";
import LoadingSvg from "../assets/loading.svg";
import { useMount } from "utils-hooks";
import "./tools";
import { getScrollPosition } from "./tools";
import Circle from "./Circle";
export var PullRefreshPosition;
(function (PullRefreshPosition) {
    PullRefreshPosition[PullRefreshPosition["top"] = 0] = "top";
    PullRefreshPosition[PullRefreshPosition["middle"] = 1] = "middle";
    PullRefreshPosition[PullRefreshPosition["bottom"] = 2] = "bottom";
})(PullRefreshPosition || (PullRefreshPosition = {}));
export var PullRefreshPositionStatus;
(function (PullRefreshPositionStatus) {
    PullRefreshPositionStatus[PullRefreshPositionStatus["notEnoughRefreshPort"] = 0] = "notEnoughRefreshPort";
    PullRefreshPositionStatus[PullRefreshPositionStatus["overRefreshPort"] = 1] = "overRefreshPort";
    PullRefreshPositionStatus[PullRefreshPositionStatus["refreshing"] = 2] = "refreshing";
})(PullRefreshPositionStatus || (PullRefreshPositionStatus = {}));
export var PullRefreshLoading;
(function (PullRefreshLoading) {
    PullRefreshLoading[PullRefreshLoading["none"] = 0] = "none";
    PullRefreshLoading[PullRefreshLoading["refreshing"] = 1] = "refreshing";
    PullRefreshLoading[PullRefreshLoading["loadData"] = 2] = "loadData";
})(PullRefreshLoading || (PullRefreshLoading = {}));
function RefreshCircle(_a) {
    var percent = _a.percent;
    return (React.createElement("div", { className: "xy-pull-refresh-indicator-refreshing" },
        React.createElement(Circle, { notransition: true, strokeWidth: 7, trailWidth: 0, percent: percent, trailColor: "transparent" })));
}
var PullRefresh = React.forwardRef(function (props, refProps) {
    var _a = props.prefixCls, prefixCls = _a === void 0 ? "xy-pull-refresh" : _a, className = props.className, style = props.style, children = props.children, _b = props.loading, loading = _b === void 0 ? false : _b, _c = props.isNoMoreData, isNoMoreData = _c === void 0 ? false : _c, _d = props.enableLoadMore, enableLoadMore = _d === void 0 ? true : _d, _e = props.enablePullRefresh, enablePullRefresh = _e === void 0 ? false : _e, _f = props.initLock, initLock = _f === void 0 ? false : _f, _g = props.threshold, threshold = _g === void 0 ? 250 : _g, onLoadMore = props.onLoadMore, onPullRefresh = props.onPullRefresh, _h = props.noMoreDataNode, noMoreDataNode = _h === void 0 ? "没有更多数据" : _h, _j = props.bottomIndicatorTips, bottomIndicatorTips = _j === void 0 ? "上滑加载数据" : _j, _k = props.notEnoughRefreshNode, notEnoughRefreshNode = _k === void 0 ? function (p) { return React.createElement(RefreshCircle, { percent: p }); } : _k, _l = props.overRefreshNode, overRefreshNode = _l === void 0 ? React.createElement(RefreshCircle, { percent: 100 }) : _l, touchStart = props.onTouchStart, touchMove = props.onTouchMove, touchEnd = props.onTouchEnd;
    var _m = useState(PullRefreshPositionStatus.notEnoughRefreshPort), pullRefreshStatus = _m[0], setPullRefreshStatus = _m[1];
    var position = useRef(PullRefreshPosition.top);
    var loadingStatus = useRef(PullRefreshLoading.none);
    var _o = useState(0), percent = _o[0], setPercent = _o[1];
    var isLoad = useRef(false);
    var start = useRef(null);
    var offset = useRef(null);
    var ref = useRef(null);
    var contentRef = useRef(null);
    var topIndicator = useRef(null);
    function onTouchStart(e) {
        start.current = e.touches[0].clientY;
        if (position.current !== PullRefreshPosition.middle) {
            offset.current = 0;
        }
        if (touchStart) {
            touchStart(e);
        }
    }
    function onTouchMove(e) {
        var target = e.target;
        var wrapper = e.currentTarget;
        var content = contentRef.current;
        var point = e.touches[0];
        var _offset = point.clientY - start.current;
        offset.current = _offset;
        target.stopScroll = true;
        // 阻止滚动条, 修复ios拖拽显示出浏览器背景问题
        if (content.scrollHeight > wrapper.offsetHeight) {
            if (position.current === PullRefreshPosition.bottom) {
                if (_offset >= 0) {
                    target.stopScroll = false;
                }
            }
            else if (position.current === PullRefreshPosition.top) {
                if (_offset < 0) {
                    target.stopScroll = false;
                }
            }
            else {
                target.stopScroll = false;
            }
        }
        if (position.current === PullRefreshPosition.top) {
            // 设置上拉指示符的状态
            if (pullRefreshStatus !== PullRefreshPositionStatus.refreshing) {
                setPercent(_offset / threshold);
                if (_offset > threshold && pullRefreshStatus !== PullRefreshPositionStatus.overRefreshPort) {
                    setPullRefreshStatus(PullRefreshPositionStatus.overRefreshPort);
                }
                if (_offset < threshold && pullRefreshStatus !== PullRefreshPositionStatus.notEnoughRefreshPort) {
                    setPullRefreshStatus(PullRefreshPositionStatus.notEnoughRefreshPort);
                }
            }
        }
        if (position.current !== PullRefreshPosition.middle) {
            // 设置上拉，下拉距离
            // Tips offset * 0.3 为了计算阻力
            setContentStyle({ transform: "translate3d(0px, " + _offset * 0.3 + "px, 0px)", transition: "none" });
        }
        if (touchMove) {
            touchMove(e, _offset);
        }
    }
    function onTouchEnd(e) {
        var target = e.currentTarget;
        var content = contentRef.current;
        var topElement = topIndicator.current;
        if (content.scrollHeight < target.offsetHeight && offset.current < 0 && Math.abs(offset.current) >= threshold) {
            loadData();
        }
        if (position.current !== PullRefreshPosition.middle) {
            if (position.current === PullRefreshPosition.bottom || offset.current <= threshold) {
                setContentStyle({ transform: "translate3d(0px, 0px, 0px)", transition: null });
                start.current = null;
            }
            else if (enablePullRefresh) {
                setContentStyle({ transform: "translate3d(0px, " + topElement.offsetHeight + "px, 0px)", transition: null });
                if (loadingStatus.current !== PullRefreshLoading.refreshing) {
                    setPullRefreshStatus(PullRefreshPositionStatus.refreshing);
                }
                start.current = null;
                refresh();
            }
        }
        else {
            setContentStyle({ transform: "translate3d(0px, 0px, 0px)", transition: null });
        }
        if (touchEnd) {
            touchEnd(e);
        }
    }
    function onScroll(e) {
        var target = e.currentTarget;
        position.current = getScrollPosition(target);
        if (position.current === PullRefreshPosition.bottom) {
            loadData();
        }
    }
    function refresh() {
        if (enablePullRefresh && loadingStatus.current !== PullRefreshLoading.refreshing && onPullRefresh) {
            loadingStatus.current = PullRefreshLoading.refreshing;
            onPullRefresh();
        }
    }
    function loadData() {
        if (enableLoadMore && loadingStatus.current !== PullRefreshLoading.loadData && !isNoMoreData && onLoadMore) {
            loadingStatus.current = PullRefreshLoading.loadData;
            onLoadMore();
        }
    }
    function renderTopIndicator() {
        if (!enablePullRefresh) {
            return null;
        }
        return (React.createElement("div", { className: classNames(prefixCls + "-indicator", "top"), ref: topIndicator },
            React.createElement("div", { className: prefixCls + "-indicator-notEnoughRefreshPort", style: { display: pullRefreshStatus === PullRefreshPositionStatus.notEnoughRefreshPort ? "block" : "none" } }, notEnoughRefreshNode instanceof Function ? notEnoughRefreshNode(percent * 100) : notEnoughRefreshNode),
            React.createElement("div", { className: prefixCls + "-indicator-overRefreshPort", style: { display: pullRefreshStatus === PullRefreshPositionStatus.overRefreshPort ? "block" : "none" } }, overRefreshNode),
            React.createElement("div", { className: prefixCls + "-indicator-refreshing", style: { display: pullRefreshStatus === PullRefreshPositionStatus.refreshing ? "block" : "none" } },
                React.createElement("img", { src: LoadingSvg, alt: "refresh data" }))));
    }
    function renderBottomIndicator() {
        return (React.createElement("div", { className: classNames(prefixCls + "-indicator", "bottom"), onClick: loadData }, loading && loadingStatus.current === PullRefreshLoading.loadData ? (React.createElement("div", { className: classNames(prefixCls + "-indicator", "bottom") },
            React.createElement("img", { src: LoadingSvg, alt: "load data" }))) : (enableLoadMore && (React.createElement("div", { className: classNames(prefixCls + "-indicator", "bottom"), onClick: loadData }, isNoMoreData ? noMoreDataNode : bottomIndicatorTips)))));
    }
    function setContentStyle(style) {
        var content = contentRef.current;
        if (content) {
            for (var key in style) {
                content.style[key] = style[key];
            }
        }
    }
    useEffect(function () {
        var timehandle;
        if (!loading && loadingStatus.current === PullRefreshLoading.refreshing) {
            setContentStyle({ transform: "translate3d(0px, 0px, 0px)", transition: null });
            timehandle = window.setTimeout(function () {
                setPullRefreshStatus(PullRefreshPositionStatus.notEnoughRefreshPort);
            }, 300);
        }
        if (!loading) {
            isLoad.current = true;
            loadingStatus.current = PullRefreshLoading.none;
            position.current = getScrollPosition(ref.current);
        }
        return function () { return window.clearTimeout(timehandle); };
    }, [loading]);
    useMount(function () {
        // 初次没有数据, 加载数据
        if (!initLock) {
            loadData();
        }
    });
    useEffect(function () {
        // 第二次 (进入页面=initLock从true改变为false)
        if (isLoad.current === false && initLock === false) {
            loadData();
        }
    }, [initLock]);
    useMount(function () {
        function stopFunc(e) {
            var target = e.target;
            if (target && target.stopScroll) {
                e.preventDefault();
            }
        }
        document.addEventListener("touchmove", stopFunc, { passive: false });
        return function () { return document.removeEventListener("touchmove", stopFunc); };
    });
    useEffect(function () {
        if (refProps) {
            if (refProps instanceof Function) {
                refProps(ref.current);
            }
            else {
                refProps.current = ref.current;
            }
        }
    }, [ref.current]);
    return (React.createElement("div", { className: classNames(prefixCls, className), style: style, onScroll: onScroll, ref: ref, onTouchStart: onTouchStart, onTouchMove: onTouchMove, onTouchEnd: onTouchEnd, onTouchCancel: onTouchEnd },
        React.createElement("div", { className: prefixCls + "-wrapper" },
            React.createElement("div", { className: prefixCls + "-content", ref: contentRef },
                renderTopIndicator(),
                React.createElement("div", null, children),
                renderBottomIndicator()))));
});
export default React.memo(PullRefresh);
//# sourceMappingURL=index.js.map